/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Ndless code.
 *
 * The Initial Developer of the Original Code is Olivier ARMAND 
 * <olivier.calc@gmail.com>.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): 
 *                 Geoffrey ANNEHEIM <geoffrey.anneheim@gmail.com>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define FIRST_RESOURCE_BYTES_SIZE	5180			// size of the first string resource: just enough to overflow the buffer- same on CAS+
#define MAX_PAYLOAD_BYTES_SIZE		FIRST_RESOURCE_BYTES_SIZE - 8 // 4: size of the function return address overwritten

static int ADDR_clean_cache[] ={0x100ae624,0x100ae7f8,0x100ae988};   // the overrun return address
static int PAYLOAD_BASE_ADDRESS[] = {0x104dfe8c,0x104dde94,0x104dbe74}; // stack address where the resource buffer is - gets popped into pc at return of cleaning cache.

void printUsage() {
	puts(
		"Usage: LoaderWrapper <OS> <infile.bin> <outfile.tns>\n"
                "<OS> has OS version 1=1.0.494,2=1.0.529, 3=1.0.554"
		"<infile> contains raw ARM code"
	); 	
}

void printErrorAndExit(const char* msg, int code) {
	printf("Error: %s\n", msg);
	exit(code);
}

void writeLittleEndian(FILE* fOut, unsigned long i) {
	/*unsigned long iEndianness = ((i & 0xff) << 24) + ((i & 0xff00) << 8)
										+ ((i & 0xff0000) >> 8) + ((i >> 24) & 0xff);*/
														
	unsigned long iEndianness = i;	// Thanks of x86 architecture
														
	fwrite(&iEndianness, sizeof(unsigned long), 1, fOut);
}

int getFileSize(FILE *f) {
	int pos;
	int end;

	pos = ftell(f);
	fseek (f, 0, SEEK_END);
	end = ftell(f);
	fseek (f, pos, SEEK_SET);

	return end;
}

void wrap(FILE* fIn, FILE* fOut, int payload_size, int os) {
	char* buffer;
	
	if (payload_size > MAX_PAYLOAD_BYTES_SIZE) {
		return;	
	}
	
	// Build a strings.res file (header)
	writeLittleEndian(fOut, 1); // number of string resources in the file
	writeLittleEndian(fOut, FIRST_RESOURCE_BYTES_SIZE); // total number of characters
	writeLittleEndian(fOut, FIRST_RESOURCE_BYTES_SIZE); // length of resource
	
	buffer = (char*)malloc(MAX_PAYLOAD_BYTES_SIZE);
	memset(buffer, 0x00, MAX_PAYLOAD_BYTES_SIZE);
	fread(buffer, payload_size, 1, fIn);
        *(unsigned*)buffer = PAYLOAD_BASE_ADDRESS[os]+4;
	fwrite(buffer, MAX_PAYLOAD_BYTES_SIZE, 1, fOut);        
	free(buffer);
	
	writeLittleEndian(fOut, PAYLOAD_BASE_ADDRESS[os]);
	writeLittleEndian(fOut, ADDR_clean_cache[os]);
}

int main(int argc, char* argv[]) {
	FILE* fileInput, *fileOutput;
	int payload_size,os;
	
	if (argc != 4) {
		printUsage();
		return -1;	
	}
	
	fileInput = fopen(argv[1], "rb");
	if (fileInput == NULL) {
		printErrorAndExit("Could not open the input file.", 1);	
	}
	
	payload_size = getFileSize(fileInput);
	if (payload_size > MAX_PAYLOAD_BYTES_SIZE) {
		printf("Error: Input file cannot be bigger than %ul bytes", MAX_PAYLOAD_BYTES_SIZE);
		fclose(fileInput);
		exit(3);
	}
	
	fileOutput = fopen(argv[2], "wb");
	if (fileOutput == NULL) {
		fclose(fileInput);
		printErrorAndExit("Could not create the output file.", 2);	
	}
	os = atoi(argv[3])-1;
	wrap(fileInput, fileOutput, payload_size, os);
	
	fclose(fileInput);
	fclose(fileOutput);
	
	printf("File '%s' successfully created!\n", argv[2]);
	
	return EXIT_SUCCESS;	
}
